/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * MainActivity.java - Main Activity
 *
 * Copyright (c) 2014 Frank Meyer - frank(at)fli4l.de
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */

package com.voice_tour.remotebutler;

import java.io.File;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.res.Resources;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Environment;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.view.ViewPager;
import android.text.InputType;
import android.util.TypedValue;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemSelectedListener;

public class MainActivity extends FragmentActivity
{
	private MyAdapter					mAdapter;
    private ViewPager					mPager;
    
    private static PageFragment[]		pagefragments;
	private static final int			MenuEditServer	= Menu.FIRST + 1;
	private static final int			MenuEditPage	= Menu.FIRST + 2;
	private static final int			MenuAbout		= Menu.FIRST + 3;
	private static final int			MenuExit		= Menu.FIRST + 4;
	private MenuItem					MenuItemEditServer;
	private MenuItem					MenuItemEditPage;
	private MenuItem					MenuItemAbout;
	private MenuItem					MenuItemExit;
	private boolean						menusCreated = false;
	Resources							r;
	public static final int				MAX_SERVERS = 8;
	public static final IrmpServer[]	irmp_server = new IrmpServer[MAX_SERVERS];
	static int							gserveridx = 0;
	
	public static File					remotebutlerpath;
	public static String				ServersXmlFile = "Servers.xml";
	public static int					nServers;

    private EditText					tvae;
    private EditText					tvke;
    private EditText					t;

    
	@Override
    public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);

		File storageDir = new File("/mnt/extSdCard/RemoteButler");					// Samsung S3

		if (storageDir.isDirectory ())
		{
			Toast.makeText(this, storageDir.getAbsolutePath() + " gefunden", Toast.LENGTH_SHORT).show();
			remotebutlerpath = storageDir;
		}
		else
		{
			remotebutlerpath = Environment.getExternalStoragePublicDirectory("RemoteButler");
			remotebutlerpath.mkdirs();
		}

		nServers = IrmpServer.loadServers (irmp_server, MAX_SERVERS, remotebutlerpath, ServersXmlFile);

		if (nServers == 0)
		{
			irmp_server[0] = new IrmpServer();
			irmp_server[0].name			= "Wohnzimmer";
			irmp_server[0].ipaddress	= "192.168.10.233";
			irmp_server[0].ipprotocol	= IrmpServer.UDP; 
			irmp_server[0].port			= 10001;

			irmp_server[1] = new IrmpServer();
			irmp_server[1].name			= "IRMP-Server2";
			irmp_server[1].ipaddress	= "192.168.10.234";
			irmp_server[1].ipprotocol	= IrmpServer.UDP;
			irmp_server[1].port			= 10001;

			irmp_server[2] = new IrmpServer();
			irmp_server[2].name			= "IRMP-Server3";
			irmp_server[2].ipaddress	= "192.168.10.235";
			irmp_server[2].ipprotocol	= IrmpServer.UDP;
			irmp_server[2].port			= 10001;

			irmp_server[3] = new IrmpServer();
			irmp_server[3].name			= "IRMP-Server4";
			irmp_server[3].ipaddress	= "192.168.10.236";
			irmp_server[3].ipprotocol	= IrmpServer.UDP;
			irmp_server[3].port			= 10001;

			irmp_server[4] = new IrmpServer();
			irmp_server[4].name			= "IRMP-Server5";
			irmp_server[4].ipaddress	= "192.168.10.237";
			irmp_server[4].ipprotocol	= IrmpServer.UDP;
			irmp_server[4].port			= 10001;
			
			irmp_server[5] = new IrmpServer();
			irmp_server[5].name			= "IRMP-Server6";
			irmp_server[5].ipaddress	= "192.168.10.238";
			irmp_server[5].ipprotocol	= IrmpServer.UDP;
			irmp_server[5].port			= 10001;
			
			irmp_server[6] = new IrmpServer();
			irmp_server[6].name			= "IRMP-Server7";
			irmp_server[6].ipaddress	= "192.168.10.239";
			irmp_server[6].ipprotocol	= IrmpServer.UDP;
			irmp_server[6].port			= 10001;
			
			irmp_server[7] = new IrmpServer();
			irmp_server[7].name			= "IRMP-Server8";
			irmp_server[7].ipaddress	= "192.168.10.240";
			irmp_server[7].ipprotocol	= IrmpServer.UDP;
			irmp_server[7].port			= 10001;
			
			nServers = 8;
		}

		pagefragments = new PageFragment[PageFragment.NUM_PAGES];
		setContentView(R.layout.mainactivity);
	    mAdapter = new MyAdapter(getSupportFragmentManager());
	    mPager = (ViewPager) findViewById(R.id.viewpager);
	    mPager.setAdapter(mAdapter);

        Toast.makeText(getBaseContext(), "Sendetasten für Makros bitte in blau belegen!", Toast.LENGTH_SHORT).show();
}

    public static class MyAdapter extends FragmentPagerAdapter
    {
        public MyAdapter(FragmentManager fm)
        {
            super(fm);
        }
 
        @Override
        public int getCount()
        {
            return PageFragment.NUM_PAGES;
        }
 
        @Override
        public Fragment getItem(int position)
        {
        	if (position < PageFragment.NUM_PAGES)
        	{
        		pagefragments[position] = PageFragment.newInstance(position);
                return pagefragments[position];
        	}
        	return null;
        }
    }
    
    public View myfindViewById(int id)
    {
        return findViewById (id);
    }

	private void myExit ()
	{
		Toast.makeText(getBaseContext(), "Bye bye...", Toast.LENGTH_SHORT).show();
		onDestroy();
		finish ();
		System.exit(0);
	}

	public  void setMenusEnabled (boolean enabled)
	{
		if (menusCreated)
		{
			MenuItemEditServer.setEnabled(enabled);
			MenuItemEditPage.setEnabled(enabled);
		}
	}

	/** create the menu items */
	public void populateMenu(Menu menu)
	{
		// menu.setQwertyMode(true);
		MenuItemEditServer	= menu.add(0, MenuEditServer,	0, "Server");
		MenuItemEditPage	= menu.add(0, MenuEditPage, 	0, "Bearbeiten");
		MenuItemAbout		= menu.add(0, MenuAbout,		0, "Über");
		MenuItemExit		= menu.add(0, MenuExit,			0, "Beenden");

		menusCreated = true;
		setMenusEnabled (true);
		MenuItemAbout.setEnabled(true);
		MenuItemExit.setEnabled(true);
	}

	/** hook into menu button for activity */
	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		populateMenu(menu);
		return super.onCreateOptionsMenu(menu);
	}

	/** when menu button option selected */
	@Override
	public boolean onOptionsItemSelected(MenuItem item)
	{
		return applyMenuChoice(item) || super.onOptionsItemSelected(item);
	}

	/** respond to menu item selection */
	public boolean applyMenuChoice(MenuItem item)
	{
		switch (item.getItemId())
		{
		    case MenuEditServer:
		    {
		    	editServer ();
				break;
		    }
		    case MenuEditPage:
		    {
	    		setEditMode (true);		    		
		    	break;
		    }

		    case MenuAbout:
		    {
		    	about ();
		    	break;
		    }

		    case MenuExit:
		    {
				myExit ();
		    	break;
		    }

		    default:
			{
				return false;
			}
		}
		return true;
	}

	public static void setEditMode (boolean edit)
	{
    	for (int idx = 0; idx < PageFragment.NUM_PAGES; idx++)
    	{
    		if (pagefragments[idx] != null)
    		{
    			pagefragments[idx].setEditMode (edit);
    		}
    	}
	}

	private void about ()
    {
    	final AlertDialog.Builder alert = new AlertDialog.Builder(this);
		alert.setTitle("Über RemoteButler");
		alert.setMessage("RemoteButler Version 1.0\n\n(C) 2012-2014 Frank Meyer");
        // alert.setIcon(R.drawable.icon);
        alert.setPositiveButton("Ok", new DialogInterface.OnClickListener()
		{
			public void onClick(DialogInterface dialog, int whichButton)
			{
			}
		});
		alert.show();
    }

	private void editServer ()
	{
		String[]			servernames;
	    final int			PADDING = 20;
	    int					widthleft;
	    int					widthright;

	    gserveridx = 0;
	    
        r = getResources();
        widthleft = (int) (TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 80, r.getDisplayMetrics()) + 0.5);
        widthright = (int) (TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 160, r.getDisplayMetrics()) + 0.5);

	    AlertDialog.Builder alertbox = new AlertDialog.Builder(this);
	    // alertbox.setMessage("Schaltfläche bearbeiten");

        ScrollView sv = new ScrollView(this);
        LinearLayout ll = new LinearLayout(this);
        ll.setOrientation(LinearLayout.VERTICAL);

        final Spinner server_sp = new Spinner(this);

        servernames = new String[nServers];
        
        for (int i = 0; i < nServers; i++)
        {
        	servernames[i] = new String (MainActivity.irmp_server[i].name);
        }

        ArrayAdapter<String> server_adapter = new ArrayAdapter<String> (this, android.R.layout.simple_spinner_item, servernames);
        server_adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        server_sp.setAdapter(server_adapter);

        server_sp.setOnItemSelectedListener(new OnItemSelectedListener()
        {
			public void onItemSelected(AdapterView<?> arg0, View arg1, int arg2, long arg3)
			{
				gserveridx = server_sp.getSelectedItemPosition();
		        t.setText(irmp_server[gserveridx].name);
		        tvae.setText(irmp_server[gserveridx].ipaddress);
		        tvke.setText(String.valueOf(irmp_server[gserveridx].port));

				// Toast.makeText(this, "Selected: [" + idx + "] ", Toast.LENGTH_SHORT).show();
			}

			public void onNothingSelected(AdapterView<?> arg0)
			{
			}
        });

        server_sp.setBackgroundColor (Color.LTGRAY);

        ll.addView(server_sp);

    	LinearLayout xlt = new LinearLayout(this);
    	xlt.setOrientation(LinearLayout.HORIZONTAL);

		TextView tv = new TextView(this);
        tv.setText("Servername:");
        tv.setTextColor(Color.WHITE);
        tv.setWidth (widthleft);
        xlt.addView(tv);

        t = new EditText(this);
        t.setText(irmp_server[gserveridx].name);
        t.setTextColor(Color.WHITE);
        t.setBackgroundColor(Color.TRANSPARENT);
        t.setWidth (widthright);
		xlt.addView(t);

		ll.addView (xlt);

    	LinearLayout xla = new LinearLayout(this);
    	xla.setOrientation(LinearLayout.HORIZONTAL);

    	TextView tva = new TextView(this);
        tva.setText("IP-Adresse:");
        tva.setTextColor(Color.WHITE);
        tva.setWidth (widthleft);
        xla.addView(tva);

        tvae = new EditText(this);
        tvae.setText(irmp_server[gserveridx].ipaddress);
        tvae.setTextColor(Color.WHITE);
        tvae.setBackgroundColor(Color.TRANSPARENT);
        tvae.setWidth (widthright);
        // tvae.setInputType(InputType.TYPE_CLASS_PHONE);
		xla.addView(tvae);

		ll.addView (xla);

    	LinearLayout xlk = new LinearLayout(this);
    	xlk.setOrientation(LinearLayout.HORIZONTAL);

    	TextView tvk = new TextView(this);
        tvk.setText("Port:");
        tvk.setTextColor(Color.WHITE);
        tvk.setWidth (widthleft);
        xlk.addView(tvk);

        tvke = new EditText(this);
        tvke.setText(String.valueOf(irmp_server[gserveridx].port));
        tvke.setTextColor(Color.WHITE);
        tvke.setBackgroundColor(Color.TRANSPARENT);
        tvke.setWidth (widthright);
        tvke.setInputType(InputType.TYPE_CLASS_NUMBER);
		xlk.addView(tvke);

		ll.addView (xlk);

        Button test = new Button(this);
        test.setTextColor (Color.WHITE);
        test.setText("Test Server");
        test.setOnClickListener(new OnClickListener()
      	{
    		@Override
    		public void onClick(View v)
    		{
    		    DatagramSocket s;
    		    String message = "P";				// ping

    		    try
    			{
    				s = new DatagramSocket();

    				String ipaddress = tvae.getText().toString().trim();
        		    int server_port = Integer.parseInt(tvke.getText().toString().trim());
    				
    				final InetAddress RemoteServer = InetAddress.getByName(ipaddress);  
    			      
    			    byte[] messageBytes = message.getBytes();  
    			    int msg_length = messageBytes.length;
    			    // int msg_length = message.length();									// falsch für Umlaute!
    			    
    			    DatagramPacket p = new DatagramPacket(messageBytes, msg_length, RemoteServer, server_port);  
    			      
    			    s.send(p);
    			    
    			    byte[] receiveData = new byte[2];

    			    s.setSoTimeout(500);   // set the timeout in millisecounds.
    	            DatagramPacket receivePacket = new DatagramPacket(receiveData, receiveData.length);
    			    
    			    try
    			    {
    			    	s.receive(receivePacket);
    			    	Toast.makeText(getBaseContext(), "Erfolgreich!", Toast.LENGTH_SHORT).show();
    			    }
    			    catch (SocketTimeoutException e)
    			    {
    			    	Toast.makeText(getBaseContext(), "Keine Antwort!", Toast.LENGTH_SHORT).show();
    	            }
   			    	s.close();
    			}
    			catch (SocketException e)
    			{
    				Toast.makeText(getBaseContext(), "Socket Exception!", Toast.LENGTH_SHORT).show();
    				// e.printStackTrace();
    			}
    		    catch (UnknownHostException e)
    			{
    				Toast.makeText(getBaseContext(), "Unknown Host!", Toast.LENGTH_SHORT).show();
    				// e.printStackTrace();
    			}
    		    catch (IOException e)
    		    {
    				Toast.makeText(getBaseContext(), "IO exception!", Toast.LENGTH_SHORT).show();
    				// e.printStackTrace();
    			}  
    		}
      	});
	
        ll.addView(test);

        sv.addView(ll);

        sv.setBackgroundColor(Color.rgb(25, 25, 25));
        int padding_px = (int) (TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, PADDING, r.getDisplayMetrics()) + 0.5);
        sv.setPadding(padding_px, 0, padding_px, 0);
        alertbox.setView (sv);

	    alertbox.setPositiveButton("Ok", new DialogInterface.OnClickListener()
	    {
	        public void onClick(DialogInterface arg0, int arg1)
	        {
				String name = t.getText().toString().trim();

				if (name.contentEquals(""))
				{
					Toast.makeText(getBaseContext(), "Name muss ausgefüllt werden!", Toast.LENGTH_SHORT).show();
				}
				else
				{
					String ipaddress = tvae.getText().toString().trim();
	
					if (ipaddress.contentEquals(""))
					{
						Toast.makeText(getBaseContext(), "IP-Adresse muss ausgefüllt werden!", Toast.LENGTH_SHORT).show();
						ipaddress = "...";
					}
					else
					{
						int port = Integer.parseInt (tvke.getText().toString().trim());
						
						irmp_server[gserveridx].name = name;
						irmp_server[gserveridx].ipaddress = ipaddress;
						irmp_server[gserveridx].port = port;
						irmp_server[gserveridx].ipprotocol = IrmpServer.UDP;		// TODO
						IrmpServer.saveServers (irmp_server, nServers, remotebutlerpath, ServersXmlFile);
					}
		        }
	        }
	    });
	
	    alertbox.setNegativeButton("Abbrechen", new DialogInterface.OnClickListener()
	    {
	        public void onClick(DialogInterface arg0, int arg1)
	        {
	            ;
	        }
	    });
	    alertbox.show ();
	}


}