/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * RemoteButton.java - Remote Button class
 *
 * Copyright (c) 2014 Frank Meyer - frank(at)fli4l.de
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */

package com.voice_tour.remotebutler;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xmlpull.v1.XmlSerializer;

import android.graphics.Color;
import android.util.Log;
import android.util.Xml;
import android.widget.Button;

public class RemoteButton extends IrmpData
{
	public static final int			BLACK			= 0;
	public static final int			BLUE			= 1;
	public static final int			GREEN			= 2;
	public static final int			CYAN			= 3;
	public static final int			RED				= 4;
	public static final int			MAGENTA			= 5;
	public static final int			YELLOW			= 6;
	public static final int			GRAY			= 7;
	public static final int			DKGRAY			= 8;
	public static final int			INVISIBLE		= 9;		// last, don't count it in BUTTON_COLORS 
	
	public static final int			BUTTON_COLORS	= 9;
	public static final int[]		MUL =
									{
										0xFF505050,
										0xFF505050,
										0xFF505050,
										0xFF505050,
										0xFF505050,
										0xFF505050,
										0xFF505050,
										0xFF505050,
										0xFF505050,
										0x00000000
									};
	public static final int[]		COL =
									{
										Color.BLACK,
										Color.BLUE,
									    Color.GREEN,
									    Color.CYAN,
									    Color.RED,
									    Color.MAGENTA,
									    Color.YELLOW,
									    Color.GRAY,
									    Color.DKGRAY,
										0x00000000
									};
	
	public	boolean					visible;
	public	Button					button;
    public	String					text;
    public	int						color;
    
    // public static  int				nbuttons;
    
    RemoteButton ()
	{
		// nbuttons++;
	}

    RemoteButton (boolean vis, String txt, int col, int srv, int prot, int addr, int cmd, int fl)
	{
		this ();
		setMembers (vis, txt, col, srv, prot, addr, cmd, fl);
	}

	private void setMembers (boolean vis, String txt, int col, int srv, int prot, int addr, int cmd, int fl)
	{
		visible		= vis;
		text		= txt;
		color		= col;
		serveridx	= srv;
		protocol	= prot;
		address		= addr;
		command		= cmd;
		flags		= fl;
	}

	public static int loadButtons (String[] titles, RemoteButton[] buttons, int pages, final int buttons_pp, File path, String filename)
	{
		int			rtc = 0;

	    class MyXMLHandler extends DefaultHandler
	    {
	    	private Boolean		currentElement = false;
	    	private String		currentValue = new String("");
			public RemoteButton[] remotebuttons;
			public int			buttons_per_page = buttons_pp;
			private	int			pageno = 0;
			private int			idx;
			private int			buttons_read = 0;
			public String[]		page_titles;
			private boolean		visible;
			private String		text;
			private int			color;
			private int			serveridx;
			private int			protocol;
			private int			address;
			private int			command;
			private int			flags;
			
	    	/** Called when tag starts ( ex:- <name>AndroidPeople</name> 
	    	 * -- <name> )*/
	    	@Override
	    	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
	    	{
	    		currentElement = true;
	
	    		if (localName.equalsIgnoreCase("page"))				// default values in section page
	    		{
	    			pageno++;										// use next page if pageno not given
	    		}
	    		else if (localName.equalsIgnoreCase("button"))		// default values
	    		{
	    			visible			= true;
	    			text			= "";
	    			color			= 2;
	    			serveridx		= 0;
	    			protocol		= 0;
	    			address			= 0;
	    			command			= 0;
	    			flags			= 0;
	    		}

    			// Toast.makeText(getBaseContext(), "start localName = " + localName + " value = " + currentValue, Toast.LENGTH_SHORT).show();
	    		// if (localName.equalsIgnoreCase("showgrad"))
	    		// {
	    		//	;															// create any object here
	    		// }

	    	}
	
	    	/** Called when tag closing ( ex:- <name>AndroidPeople</name> 
	    	 * -- </name> )*/
	    	@Override
	    	public void endElement(String uri, String localName, String qName) throws SAXException
	    	{
	    		currentElement = false;

	    		String value = currentValue.trim();
	    		// Toast.makeText(getBaseContext(), "end localName = " + localName + " value = " + currentValue, Toast.LENGTH_SHORT).show();
	    		if (localName.equalsIgnoreCase("pageno"))
	    		{
	    			pageno = Integer.parseInt(value);
	    		}
	    		else if (localName.equalsIgnoreCase("idx"))
	    		{
	    			idx = Integer.parseInt(value);
	    		}
	    		else if (localName.equalsIgnoreCase("text"))
	    		{
	    			text = value;
	    		}
	    		else if (localName.equalsIgnoreCase("color"))
	    		{
	    			color = Integer.parseInt(value);
	    		}
	    		else if (localName.equalsIgnoreCase("serveridx"))
	    		{
	    			serveridx = Integer.parseInt(value) ;
	    		}
	    		else if (localName.equalsIgnoreCase("protocol"))
	    		{
	    			protocol = Integer.parseInt(value) ;
	    		}
	    		else if (localName.equalsIgnoreCase("address"))
	    		{
	    			address = Integer.parseInt(value) ;
	    		}
	    		else if (localName.equalsIgnoreCase("command"))
	    		{
	    			command = Integer.parseInt(value) ;
	    		}
	    		else if (localName.equalsIgnoreCase("flags"))
	    		{
	    			flags = Integer.parseInt(value) ;
	    		}
	    		else if (localName.equalsIgnoreCase("button"))
	    		{
	    			remotebuttons[pageno * buttons_per_page + idx] = new RemoteButton (visible, text, color, serveridx, protocol, address, command, flags);
	    			buttons_read++;
	    		}
	    		else if (localName.equalsIgnoreCase("title"))
	    		{
	    			page_titles[pageno] = new String (value);
	    		}

	    		// fm: empty currentValue!
	    		// if there is an empty value, currentValue will not change, but contains last(!) value
	    		currentValue = new String("");
	    	}
	
	    	/** Called to get tag characters ( ex:- <name>AndroidPeople</name> 
	    	 * -- to get AndroidPeople Character ) */
	    	/* fm: this method can be called multiple for the same tag! */
	    	@Override
	    	public void characters(char[] ch, int start, int length) throws SAXException
	    	{
	    		if (currentElement)
	    		{
    				currentValue += new String(ch, start, length);
	    		}
	    	}
	    }
	
		try
		{
			/** Handling XML */
			SAXParserFactory spf = SAXParserFactory.newInstance();
			SAXParser sp = spf.newSAXParser();
			XMLReader xr = sp.getXMLReader();
	
			/** Create handler to handle XML Tags ( extends DefaultHandler ) */
			MyXMLHandler myXMLHandler = new MyXMLHandler();
			myXMLHandler.remotebuttons = buttons;
			myXMLHandler.page_titles = titles;
			xr.setContentHandler(myXMLHandler);
	
			// URL version:
			// URL sourceUrl = new URL("http://www.androidpeople.com/wp-content/uploads/2010/06/example.xml");
			// xr.parse(new InputSource(sourceUrl.openStream()));
			//
			// File version:
			File file = new File(path, filename);
			InputStream fIn = new FileInputStream(file);
			xr.parse(new InputSource(fIn));
			rtc = myXMLHandler.buttons_read;
		}
		catch (Exception e)
		{
			// Toast.makeText(getApplicationContext(), "XML Error: " + e, Toast.LENGTH_SHORT).show();
        	Log.e("Exception", "exception in loadLocations() method");
		}

		return rtc;
	}

	public static boolean saveButtons (String[] titles, RemoteButton[] buttons, int pages, int buttons_per_page, File path, String filename)
    {
		boolean		rtc = false;
		int			pageno;
		int			idx;
		int			i;
        File		newxmlfile = new File(path, filename);

        try
        {
        	newxmlfile.createNewFile();
        }
        catch(IOException e)
        {
        	Log.e("IOException", "exception in saveLocations() method");
        }

        FileOutputStream fileos = null;       	

        try
        {
        	fileos = new FileOutputStream(newxmlfile);
        }
        catch(FileNotFoundException e)
        {
        	Log.e("FileNotFoundException", "cannot create FileOutputStream");
        }

        XmlSerializer serializer = Xml.newSerializer();

        try
        {
			serializer.setOutput(fileos, "UTF-8");
			serializer.startDocument(null, Boolean.valueOf(true)); 
			serializer.setFeature("http://xmlpull.org/v1/doc/features.html#indent-output", true); 

			serializer.startTag(null, "RemoteButler");
			{
				for (pageno = 0; pageno < pages; pageno++)
				{
					serializer.startTag(null, "page");
					{
						serializer.startTag(null, "pageno");
						serializer.text(String.valueOf(pageno));
						serializer.endTag(null, "pageno");

						serializer.startTag(null, "title");
						serializer.text(titles[pageno]);
						serializer.endTag(null, "title");

						for (idx = 0; idx < buttons_per_page; idx++)
						{
							i = pageno * buttons_per_page + idx;
	
							if (buttons[i].visible)
							{
								serializer.startTag(null, "button");
								{
									serializer.startTag(null, "idx");
									serializer.text(String.valueOf(idx));
									serializer.endTag(null, "idx");
	
									serializer.startTag(null, "text");
	
									if (buttons[i].text == null)
									{
										serializer.text("");
									}
									else
									{
										serializer.text(buttons[i].text);
									}
									
									serializer.endTag(null, "text");
					
									serializer.startTag(null, "color");
									serializer.text(String.valueOf(buttons[i].color));
									serializer.endTag(null, "color");
					
									serializer.startTag(null, "serveridx");
									serializer.text(String.valueOf(buttons[i].serveridx));
									serializer.endTag(null, "serveridx");
					
									serializer.startTag(null, "protocol");
									serializer.text(String.valueOf(buttons[i].protocol));
									serializer.endTag(null, "protocol");
					
									serializer.startTag(null, "address");
									serializer.text(String.valueOf(buttons[i].address));
									serializer.endTag(null, "address");
					
									serializer.startTag(null, "command");
									serializer.text(String.valueOf(buttons[i].command));
									serializer.endTag(null, "command");
					
									serializer.startTag(null, "flags");
									serializer.text(String.valueOf(buttons[i].flags));
									serializer.endTag(null, "flags");
					
									serializer.endTag(null, "button");
								}
							}
						}
						serializer.endTag(null, "page");
					}
				}
			}
			serializer.endTag(null, "RemoteButler");
			serializer.endDocument();
			serializer.flush();
			fileos.close();
			rtc = true;
		}
        catch (Exception e)
        {
			Log.e("Exception","error occurred while creating xml file");
		}

        return rtc;
    }

}
